from datetime import datetime

from confluent_kafka.avro import AvroConsumer, SerializerError

import pdp_pipeline_core.core as core
from pdp_pipeline_core.core import logger, exception_logger


class PdpConfluentConsumer:
    def get_consumer(self):
        return self._consumer

    def __init__(self, bootstrap_servers, schema_registry_url, group_id):
        # handle common configs
        conf = {'bootstrap.servers': bootstrap_servers, 'schema.registry.url': schema_registry_url,
                'group.id': group_id, 'auto.offset.reset': "earliest", 'enable.auto.commit': False}
        self._consumer = AvroConsumer(conf)

    def consume(self, topic):
        c = self._consumer
        c.subscribe([topic])
        logger.info("consumer started")
        while True:
            try:
                msg = c.poll(1)

                # There were no messages on the queue, continue polling
                if msg is None:
                    continue

                if msg.error():
                    print("Consumer error: {}".format(msg.error()))
                    continue

                logger.debug(f'Received message {msg}')
                # noinspection PyDictCreation
                message = {"offset": msg.offset(), "partition": msg.partition(), "timestamp": msg.timestamp(),
                           "topic": msg.topic(), "headers": msg.headers(),
                           "value": msg.value()['data']}

                self.read_metadata(message, msg)

                self.update_core_batch_id(message)

                yield message, msg

            except SerializerError as e:
                # Report malformed record, discard results, continue polling
                exception_logger.exception(f"Message deserialization failed {e.args}", e)
                continue

            except Exception as e:
                # Report malformed record, discard results, continue polling
                exception_logger.exception(f"Message consumption failed with exception {e.args}", e)
                continue

        print("Shutting down consumer..")
        c.close()

    @staticmethod
    def update_core_batch_id(message):
        core.batch_id = message["batchid"]

    @staticmethod
    def read_metadata(message, msg):
        if msg.headers() is None:
            message["batchid"] = "No BatchID"
            message["scenarioid"] = "No ScenarioID"
            message["dataset"] = "dataset name unavailable"
        elif len(msg.headers()) > 0:
            for key, value in msg.headers():
                message[key] = str(value, 'utf-8')
