import json
import os
import time

import pandas as pd
import requests as requests
from confluent_kafka import avro
from confluent_kafka.avro import AvroProducer
from pdp_pipeline_core.core import logger, exception_logger



class PdpConfluentProducer:
    key_schema_str = """
    {
       "namespace": "my.different_countries",
       "name": "key",
       "type": "record",
       "fields" : [
         {
           "name" : "name",
           "type" : "string"
         }
       ]
    }
    """
    def __init__(self, bootstrap_servers,topic,schema_registry_url):
        key_schema = avro.loads(self.key_schema_str)
        get_schema_req_data = requests.get(f"{schema_registry_url}/subjects/{topic}-value/versions/latest")
        get_schema_req_data.raise_for_status()
        schema_string = get_schema_req_data.json()['schema']
        value_schema = avro.loads(schema_string)
        self._producer = AvroProducer({
            'bootstrap.servers': bootstrap_servers,
            'schema.registry.url': f'{schema_registry_url}'
        }, default_key_schema=key_schema, default_value_schema=value_schema)

    @staticmethod
    def read_csv(path):
        df = pd.read_csv(path)
        df = df.where(pd.notnull(df), "N/A")
        input_records = df.to_dict(orient='records')
        return input_records

    def get_data(self, input_file, message_format):
        if message_format == "csv":
            array_dict = self.read_csv(input_file)
            return array_dict
        else:
            if message_format == 'json':
                with open(input_file, 'r') as input_data_file:
                    records = json.loads(input_data_file.read())
                return records

    @staticmethod
    def on_send_success(record_metadata):
        logger.info("successfully published!")
        logger.info(record_metadata.topic)
        logger.info(record_metadata.partition)
        logger.info(record_metadata.offset)

    @staticmethod
    def on_send_error(exception):
        exception_logger.exception(exception.args)

    def publish(self, topic, input_file, message_format):
        data = self.get_data(input_file, message_format)
        key = {"name": "Key"}
        value = {'data':list(data)}
        self._producer.produce(topic=topic, value=value,key=key)
        time.sleep(1)
        self._producer.flush()
        os.remove(input_file)
